import { ComponentHarness, HarnessQuery, ContentContainerComponentHarness, HarnessLoader } from '@angular/cdk/testing';
import { AbstractType, InjectionToken, Type, Signal } from '@angular/core';
import { TestModuleMetadata, ComponentFixture } from '@angular/core/testing';
import { Deferred } from '@s-libs/js-core';
import { TestRequest } from '@angular/common/http/testing';
import { HttpRequest } from '@angular/common/http';
import { TestScheduler } from 'rxjs/testing';

/**
 * Provides the foundation for an opinionated testing pattern.
 * - All tests are run in the `fakeAsync` zone. This gives you full control over
 *   the timing of everything by default.
 * - Variables that are initialized for each test exist in a context that is
 *   thrown away, so they cannot leak between tests.
 * - Clearly separates initialization code from the test itself.
 * - Gives control over the simulated date & time with a single line of code.
 * - Automatically includes {@link https://angular.dev/api/common/http/testing/provideHttpClientTesting | provideHttpClientTesting()} to stub network requests without additional setup.
 * - Always verifies that no unexpected http requests were made.
 * - Automatically discards periodic tasks and flushes pending timers at the end of each test to avoid the error "X timer(s) still in the queue".
 *
 * This example tests a simple service that uses `HttpClient`, and is tested by using `AngularContext` directly. More often `AngularContext` will be used as a super class. See {@link ComponentContext} for more common use cases.
 *
 * ```ts
 * // This is the class we will test.
 * @Injectable({ providedIn: 'root' })
 * class MemoriesService {
 *   #httpClient = inject(HttpClient);
 *
 *   getLastYearToday(): Observable<any> {
 *     const datetime = new Date();
 *     datetime.setFullYear(datetime.getFullYear() - 1);
 *     const date = datetime.toISOString().split('T')[0];
 *     return this.#httpClient.get(`http://example.com/post-from/${date}`);
 *   }
 * }
 *
 * describe('MemoriesService', () => {
 *   // Tests should have exactly 1 variable outside an "it": `ctx`.
 *   let ctx: AngularContext;
 *   beforeEach(() => {
 *     ctx = new AngularContext({ providers: [provideHttpClient()] });
 *   });
 *
 *   it('requests a post from 1 year ago', () => {
 *     // Before calling `run`, set up any context variables this test needs.
 *     ctx.startTime = new Date('2004-02-16T10:15:00.000Z');
 *
 *     // Pass the test itself as a callback to `run()`.
 *     ctx.run(() => {
 *       const httpBackend = ctx.inject(HttpTestingController);
 *       const myService = ctx.inject(MemoriesService);
 *
 *       myService.getLastYearToday().subscribe();
 *
 *       httpBackend.expectOne('http://example.com/post-from/2003-02-16');
 *     });
 *   });
 * });
 * ```
 */
declare class AngularContext {
    #private;
    /**
     * Set this before calling `run()` to mock the time at which the test starts.
     */
    startTime: Date;
    /**
     * @param moduleMetadata passed along to {@linkcode https://angular.dev/api/core/testing/TestBedStatic#configureTestingModule | TestBed.configureTestingModule()}. Automatically includes {@link HttpClientTestingModule} for you.
     */
    constructor(moduleMetadata?: TestModuleMetadata);
    /**
     * Returns the current `AngularContext` that is in use, or `undefined` if there is not one. A context is defined to be "in use" from the time it is constructed until after its `run()` method completes.
     */
    static getCurrent(): AngularContext | undefined;
    /**
     * Runs `test` in a `fakeAsync` zone. It can use async/await, but be sure anything you `await` is already due to execute (e.g. if a timeout is due in 3 seconds, call `.tick(3000)` before `await`ing its result).
     *
     * Also runs the following in this order, all within the same zone:
     *
     * 1. `this.init()`
     * 2. `test()`
     * 3. `this.verifyPostTestConditions()`
     * 4. `this.cleanUp()`
     */
    run(test: () => Promise<void> | void): void;
    /**
     * Returns whether this context is currently executing the {@linkcode #run} callback.
     */
    isRunning(): boolean;
    /**
     * Gets a service or other injectable from the root injector.
     *
     * This implementation is a simple pass-through to {@linkcode https://angular.dev/api/core/testing/TestBedStatic#inject | TestBed.inject()}, but subclasses may provide their own implementation. It is recommended to use this in your tests instead of using `TestBed` directly.
     */
    inject<T>(token: AbstractType<T> | InjectionToken<T> | Type<T>): T;
    /**
     * Returns whether any components match the given `query`.
     */
    hasHarness<H extends ComponentHarness>(query: HarnessQuery<H>): Promise<boolean>;
    /**
     * Gets a component harness, wrapped for use in a fakeAsync test. Throws an error if no matching component is found.
     */
    getHarness<H extends ComponentHarness>(query: HarnessQuery<H>): Promise<H>;
    /**
     * Gets a component harness, wrapped for use in a fakeAsync test. Returns `null` if no matching component is found.
     */
    getHarnessOrNull<H extends ComponentHarness>(query: HarnessQuery<H>): Promise<H | null>;
    /**
     * Gets all component harnesses that match the query, wrapped for use in a fakeAsync test.
     */
    getAllHarnesses<H extends ComponentHarness>(query: HarnessQuery<H>): Promise<H[]>;
    /**
     * Advance time and trigger change detection. It is common to call this with no arguments to trigger change detection without advancing time.
     *
     * @param unit The unit of time `amount` represents. Accepts anything described in `@s-libs/s-core`'s [TimeUnit]{@linkcode https://simontonsoftware.github.io/s-js-utils/typedoc/enums/timeunit.html} enum.
     */
    tick(amount?: number, unit?: string): void;
    /**
     * This is a hook for subclasses to override. It is called during `run()`, before the `test()` callback. This implementation does nothing, but if you override this it is still recommended to call `super.init()` in case this implementation does something in the future.
     */
    protected init(): void;
    protected runChangeDetection(): void;
    /**
     * Runs post-test verifications. This base implementation runs {@linkcode https://angular.dev/api/common/http/testing/HttpTestingController#verify | HttpTestingController.verify} and {@linkcode MockErrorHandler.verify}. Unlike {@linkcode #cleanUp}, it is OK for this method to throw an error to indicate a violation.
     */
    protected verifyPostTestConditions(): void;
    /**
     * Performs any cleanup needed at the end of each test. This base implementation calls {@linkcode https://angular.dev/api/core/testing/discardPeriodicTasks | discardPeriodicTasks} and {@linkcode https://angular.dev/api/core/testing/flush | flush} to avoid an error from the `fakeAsync` zone.
     */
    protected cleanUp(): void;
}

type Inputs<T> = {
    [K in keyof T]?: T[K] extends Signal<infer U> ? U : T[K];
};

/**
 * Provides the foundation for an opinionated pattern for component tests.
 *
 * - Includes all features from {@link AngularContext}
 * - Automatically creates your component at the beginning of `run()`.
 * - Sets up Angular to call `ngOnChanges()` like it would in production. This is not the case if you use the standard `TestBed.createComponent()` directly.
 * - Wraps your component in a parent that you can easily style however you like.
 * - Lets you use {@link https://material.angular.dev/cdk/testing/overview | component harnesses} in the `fakeAsync` zone, which is normally a challenge.
 * - Automatically disables animations.
 * - Causes async {@link https://angular.dev/api/core/APP_INITIALIZER | APP_INITIALIZER}s to complete before instantiating the component. Two caveats:
 *   - this requires all work in your initializers to complete with a call to `tick()`
 *   - this requires delaying app initialization until inside the `fakeAsync` zone, i.e. with the callback to {@link #run}. If you have async initializers, you must be careful not to do things that finalize the app setup before then, such as {@link #inject}.
 *
 * A very simple example:
 * ```ts
 * @Component({ standalone: true, template: 'Hello, {{name()}}!' })
 * class GreeterComponent {
 *   readonly name = input.required<string>();
 * }
 *
 * it('greets you by name', () => {
 *   const ctx = new ComponentContext(GreeterComponent);
 *   ctx.assignInputs({ name: 'World' });
 *   ctx.run(() => {
 *  expect(ctx.fixture.nativeElement.textContent).toBe('Hello, World!');
 *   });
 * });
 * ```
 *
 * A full example, with routing and a component harness. This is the full code for a tiny Angular app:
 * ```ts
 *  /////////////////
 *  // app-context.ts
 *
 *  // To re-use your context setup, make a subclass of ComponentContext to import into any spec
 *  class AppContext extends ComponentContext<AppComponent> {
 *    constructor() {
 *      // Import `appConfig` from `app.config.ts`
 *      super(AppComponent, appConfig);
 *    }
 *  }
 *
 *  ////////////////////////
 *  // app.component.spec.ts
 *
 *  describe('AppComponent', () => {
 *    let ctx: AppContext;
 *    beforeEach(() => {
 *      ctx = new AppContext();
 *    });
 *
 *    it('can navigate to the first page', () => {
 *      ctx.run(async () => {
 *        const app = await ctx.getHarness(AppComponentHarness);
 *        await app.navigateToFirstPage();
 *        expect(ctx.fixture.nativeElement.textContent).toContain(
 *          'First works!',
 *        );
 *      });
 *    });
 *  });
 *
 *  ///////////////////////////
 *  // app.component.harness.ts
 *
 *  // A simple component harness to demonstrate its integration with component contexts
 *  class AppComponentHarness extends ComponentHarness {
 *    static hostSelector = 'app-root';
 *
 *    #getFirstPageLink = this.locatorFor('a');
 *
 *    async navigateToFirstPage(): Promise<void> {
 *      const link = await this.#getFirstPageLink();
 *      await link.click();
 *    }
 *  }
 *
 *  /////////////////////
 *  // first.component.ts
 *
 *  // A minimal component for demonstration purposes
 *  @Component({ standalone: true, template: '<p>First works!</p>' })
 *  class FirstComponent {}
 *
 *  ///////////////////
 *  // app.component.ts
 *
 *  // A minimal app component with routing for demonstration purposes
 *  @Component({
 *    selector: 'app-root',
 *    standalone: true,
 *    imports: [RouterOutlet, RouterLink],
 *    template: `
 *      <a routerLink="/first-page">First Page</a>
 *      <router-outlet />
 *    `,
 *  })
 *  class AppComponent {}
 *
 *  ////////////////////////
 *  // app.routes.ts
 *
 *  const routes: Routes = [{ path: 'first-page', component: FirstComponent }];
 *
 *  ////////////////
 *  // app.config.ts
 *
 *  const appConfig: ApplicationConfig = { providers: [provideRouter(routes)] };
 * ```
 */
declare class ComponentContext<T> extends AngularContext {
    #private;
    /**
     * The {@link ComponentFixture} for a synthetic wrapper around your component. Available with the callback to `run()`.
     */
    fixture: ComponentFixture<unknown>;
    /**
     * @param componentType `run()` will create a component of this type before running the rest of your test.
     * @param moduleMetadata passed along to {@linkcode https://angular.dev/api/core/testing/TestBedStatic#configureTestingModule | TestBed.configureTestingModule()}. Automatically includes {@link NoopAnimationsModule}, in addition to those provided by {@link AngularContext}.
     * @param unboundInputs By default a synthetic parent component will be created that binds to all your component's inputs. Pass input names here that should NOT be bound. This is useful e.g. to test the default value of an input.
     */
    constructor(componentType: Type<T>, moduleMetadata?: TestModuleMetadata, unboundInputs?: Array<keyof T>);
    /**
     * Assign css styles to the div wrapping your component. Can be called before or during `run()`. Accepts an object with the same structure as the {@link https://angular.dev/api/common/NgStyle | ngStyle directive}.
     *
     * ```ts
     * ctx.assignWrapperStyles({
     *   width: '400px',
     *   height: '600px',
     *   margin: '20px auto',
     *   border: '1px solid',
     * });
     * ```
     */
    assignWrapperStyles(styles: Record<string, any>): void;
    /**
     * Assign inputs passed into your component. Can be called before `run()` to set the initial inputs, or within `run()` to update them and trigger all the appropriate change detection and lifecycle hooks.
     */
    assignInputs(inputs: Inputs<T>): void;
    /**
     * Use within `run()` to get your instantiated component that is on the page.
     */
    getComponentInstance(): T;
    /**
     * Constructs and initializes your component. Called during `run()` before it executes the rest of your test. Runs in the same `fakeAsync` zone as the rest of your test.
     */
    protected init(): void;
    protected runChangeDetection(): void;
    /**
     * Performs any cleanup needed at the end of each test. This implementation destroys {@link fixture} and calls the super implementation.
     */
    protected cleanUp(): void;
}

/**
 * Use to control {@link isPageVisible$()} in tests. Create only one per test before anything calls `isPageVisible$()`.
 *
 * ```ts
 * const isPageVisibleHarness = new IsPageVisibleHarness();
 * isPageVisibleHarness.setVisible(false);
 *
 * const next = jasmine.createSpy();
 * isPageVisible$().subscribe(next);
 * expect(next).toHaveBeenCalledWith(false);
 *
 * isPageVisibleHarness.setVisible(true);
 * expect(next).toHaveBeenCalledWith(true);
 * ```
 *
 * It also stubs `document.visibilityState` to match.
 * ```ts
 * const isPageVisibleHarness = new IsPageVisibleHarness();
 * expect(document.visibilityState).toBe('visible');
 *
 * isPageVisibleHarness.setVisible(false);
 * expect(document.visibilityState).toBe('hidden');
 * ```
 */
declare class IsPageVisibleHarness {
    #private;
    constructor();
    /**
     * Sets the page's visibility state, and triggers any subscriptions to `isPageVisible$()`. Automatically triggers change detection if running with an {@linkcode AngularContext}.
     */
    setVisible(visible: boolean): void;
}

type ResolveType<F> = F extends (...args: any[]) => Promise<infer U> ? U : never;

/**
 * A mock method call that was made and is ready to be answered. This interface allows access to the underlying <code>jasmine.CallInfo</code>, and allows resolving or rejecting the asynchronous call's result.
 */
declare class TestCall<F extends jasmine.Func> {
    private deferred;
    private autoTick;
    /** The underlying jasmine call object */
    callInfo: jasmine.CallInfo<F>;
    constructor(deferred: Deferred<ResolveType<F>>, autoTick: boolean);
    /**
     * Resolve the call with the given value.
     */
    flush(value: ResolveType<F>): void;
    /**
     * Reject the call with the given reason.
     */
    error(reason: unknown): void;
    private maybeTick;
}

type AsyncFunc = (...args: any[]) => Promise<any>;
type AsyncMethodKeys<T> = {
    [k in keyof T]: T[k] extends AsyncFunc ? k : never;
}[keyof T];
type AsyncMethod<WrappingObject, FunctionName extends keyof WrappingObject> = WrappingObject[FunctionName] extends (...args: any[]) => Promise<any> ? WrappingObject[FunctionName] : never;
type Match<WrappingObject, FunctionName extends AsyncMethodKeys<WrappingObject>> = Parameters<AsyncMethod<WrappingObject, FunctionName>> | ((callInfo: jasmine.CallInfo<AsyncMethod<WrappingObject, FunctionName>>) => boolean);
/**
 * Controller to be used in tests, that allows for mocking and flushing any asynchronous function. For example, to mock the browser's paste functionality:
 *
 * ```ts
 *  it('can paste', () => {
 *   const clipboard = navigator.clipboard;
 *   const ctx = new AngularContext();
 *
 *   // mock the browser API for pasting
 *   const controller = new AsyncMethodController(clipboard, 'readText');
 *   ctx.run(() => {
 *     // BEGIN production code that copies to the clipboard
 *     let pastedText: string;
 *     clipboard.readText().then((text) => {
 *       pastedText = text;
 *     });
 *     // END production code that copies to the clipboard
 *
 *     controller.expectOne([]).flush('mock clipboard contents');
 *
 *     // BEGIN expect the correct results after a successful copy
 *     expect(pastedText!).toBe('mock clipboard contents');
 *     // END expect the correct results after a successful copy
 *   });
 * });
 * ```
 */
declare class AsyncMethodController<WrappingObject, FunctionName extends AsyncMethodKeys<WrappingObject>> {
    #private;
    /**
     * If you are using an `AngularContext`, the default behavior is to automatically call `.tick()` after each `.flush()` and `.error()` to trigger promise handlers and changed detection. This is the normal production behavior of asynchronous browser APIs. However, if zone.js does not patch the function you are stubbing, change detection would not run automatically. In that case you many want to turn off this behavior by passing the option `autoTick: false`. See the list of functions that zone.js patches [here](https://github.com/angular/angular/blob/master/packages/zone.js/STANDARD-APIS.md).
     */
    constructor(obj: WrappingObject, methodName: FunctionName, { autoTick }?: {
        autoTick?: boolean | undefined;
    });
    /**
     * Expect that a single call was made that matches the given parameters or predicate, and return its mock. If no such call was made, or more than one, fail with a message including `description`, if provided.
     */
    expectOne(match: Match<WrappingObject, FunctionName>, description?: string): TestCall<AsyncMethod<WrappingObject, FunctionName>>;
    /**
     * Expect that no calls were made which match the given parameters or predicate. If a matching call was made, fail with a message including `description`, if provided.
     */
    expectNone(match: Match<WrappingObject, FunctionName>, description?: string): void;
    /**
     * Search for calls that match the given parameters or predicate, without any expectations.
     */
    match(match: Match<WrappingObject, FunctionName>): Array<TestCall<AsyncMethod<WrappingObject, FunctionName>>>;
    /**
     * Verify that no unmatched calls are outstanding. If any are, fail with a message indicating which calls were not matched.
     */
    verify(): void;
}

/**
 * Creates a new object with jasmine spies for each method in `type`.
 *
 * ```ts
 * class Greeter {
 *   greet(name: string) {
 *     return `Hello, ${name}!`;
 *   }
 * }
 *
 * const spyObject = createSpyObject(Greeter);
 * spyObject.greet.and.returnValue("Hello, stub!");
 * expect(spyObject.greet("Eric")).toBe("Hello, stub!");
 * expectSingleCallAndReset(spyObject.greet, "Eric");
 * ```
 */
declare function createSpyObject<T>(type: Type<T>): jasmine.SpyObj<T>;

/**
 * Expects the spy to have been called the exact number of times as the argument lists provided, with those arguments, then resets the spy.
 *
 * ```ts
 * const spy = jasmine.createSpy();
 *
 * spy(1)
 * spy(2, 3)
 * spy()
 * expectCallsAndReset(spy, [1], [2, 3], []);
 * ```
 */
declare function expectCallsAndReset(spy: jasmine.Spy, ...allArgs: any[][]): void;

/**
 * Expects exactly one call to have been made to a jasmine spy, for it to have received the given arguments, then resets the spy.
 *
 * ```ts
 * const spy = jasmine.createSpy();
 *
 * spy(1, 2);
 * expectSingleCallAndReset(spy, 1, 2); // pass
 * expectSingleCallAndReset(spy, 1, 2); // fail
 *
 * spy(3);
 * spy(4);
 * expectSingleCallAndReset(spy, 3); // fail
 * ```
 */
declare function expectSingleCallAndReset(spy: jasmine.Spy, ...params: unknown[]): void;

/**
 * A class very similar to Angular's {@linkcode https://angular.dev/api/common/http/testing/TestRequest | TestRequest} for use with an {@linkcode AngularContext}. If you are using an `AngularContext`, this will trigger change detection automatically after you flush a response, like production behavior.
 *
 * Though it is possible to construct yourself, normally an instance of this class is obtained from {@linkcode expectRequest()}.
 *
 * ```ts
 * const ctx = new AngularContext({ providers: [provideHttpClient()] });
 * ctx.run(() => {
 *   inject(HttpClient)
 *     .get('http://example.com', { params: { key: 'value' } })
 *     .subscribe();
 *   const request = expectRequest<string>('GET', 'http://example.com', {
 *     params: { key: 'value' },
 *   });
 *   request.flush('my response body');
 * });
 * ```
 */
declare class SlTestRequest<Body extends HttpBody> {
    private req;
    /**
     * The underlying {@linkcode https://angular.dev/api/common/http/testing/TestRequest | TestRequest} object from Angular.
     */
    request: HttpRequest<any>;
    constructor(req: TestRequest);
    /**
     * Resolve the request with the given body and options, like {@linkcode https://angular.dev/api/common/http/testing/TestRequest#flush | TestRequest.flush()}.
     */
    flush(body: Body, opts?: Parameters<TestRequest['flush']>[1]): void;
    /**
     * Convenience method to flush an error response.
     */
    flushError(status?: number, { statusText, body, }?: {
        statusText?: string;
        body?: HttpBody;
    }): void;
    /**
     * Returns whether the request has been cancelled.
     */
    isCancelled(): boolean;
    private tickIfPossible;
}

type HttpMethod = 'DELETE' | 'GET' | 'POST' | 'PUT';
type HttpBody = Parameters<TestRequest['flush']>[0];
interface RequestOptions {
    params?: Record<string, string>;
    headers?: Record<string, string>;
}
/**
 * This convenience function is similar to {@linkcode https://angular.dev/api/common/http/testing/HttpTestingController | HttpTestingController.expectOne()}, with extra features. The returned request object will automatically trigger change detection when you flush a response, just like in production.
 *
 * This function is opinionated in that you must specify all aspects of the request to match. E.g. if the request specifies headers, you must also specify them in the arguments to this method.
 *
 * This function only works when you are using an {@linkcode AngularContext}.
 *
 * ```ts
 * const ctx = new AngularContext({ providers: [provideHttpClient()] });
 * ctx.run(() => {
 *   inject(HttpClient)
 *     .get('http://example.com', { params: { key: 'value' } })
 *     .subscribe();
 *   const request = expectRequest<string>('GET', 'http://example.com', {
 *     params: { key: 'value' },
 *   });
 *   request.flush('my response body');
 * });
 * ```
 */
declare function expectRequest<ResponseBody extends HttpBody>(method: HttpMethod, url: string, options?: RequestOptions & {
    body?: HttpBody;
}): SlTestRequest<ResponseBody>;

/**
 * Provides some shorthand utilities that component harnesses may want.
 */
declare class ComponentHarnessSuperclass extends ContentContainerComponentHarness {
    /**
     * Searches for an instance of the component corresponding to the given harness type under the document root element, and returns a {@link ComponentHarness} for that instance. If multiple matching components are found, a harness for the first one is returned. If no matching component is found, an error is thrown.
     */
    protected getTopLevelHarness<T extends ComponentHarness>(predicate: HarnessQuery<T>): Promise<T>;
    /**
     * Searches for all instances of the component corresponding to the given harness type under the document root element, and returns a list {@link ComponentHarness} for each instance.
     */
    protected getAllTopLevelHarnesses<T extends ComponentHarness>(predicate: HarnessQuery<T>): Promise<T[]>;
    /**
     * Gets a {@link HarnessLoader} for the document root element. This loader can be used for elements that a component creates outside its own root element (e.g. by appending to document.body).
     */
    protected getTopLevelLoader(): Promise<HarnessLoader>;
}

/**
 * Designed for temporary use when a test using `fakeAsync` is giving you "Error: X timer(s) still in the queue", and you don't know what they are. Call this at the top of the test to see everything that adds a timer printed to the JS console. In Chrome, you can click the function in the console output to be taken directly to its definition in the Sources tab.
 *
 * Be sure to call this _inside_ the test, not before, to get the most useful output.
 *
 * @returns a function to stop the logging
 *
 * ```ts
 * it('definitely does not set any timeouts', fakeAsync(() => {
 *   const stopLogging = logTimers();
 *
 *   doTheRestOfMyTest();
 *
 *   stopLogging();
 * }));
 * ```
 */
declare function logTimers(): () => void;

declare global {
    interface SymbolConstructor {
        readonly observable: symbol;
    }
}
type Run = Parameters<TestScheduler['run']>[0];
type Helpers = Parameters<Run>[0];
type Callback<T> = (testHelpers: Helpers & {
    testScheduler: TestScheduler;
}) => T;
/**
 * A helper to take some boilerplate out of RxJS's {@link https://rxjs-dev.firebaseapp.com/guide/testing/marble-testing marble testing} API.
 *
 * ```ts
 * it("doubles numbers", marbleTest(({ cold, expectObservable }) => {
 *   const source = cold("-1-2-3-|");
 *   const expected = "   -2-4-6-|");
 *   expectObservable(source.pipe(double())).toBe(expected);
 * }));
 * ```
 */
declare function marbleTest<T>(callback: Callback<T>): () => T;

/**
 * Use this when you want to write test code that doesn't actually run, instead relying only on your static tools like Typescript or a linter to raise errors.
 *
 * ```ts
 * function reject<T>(array: T[], predicate: (value: T) => boolean): T[] {
 *   return array.filter((value) => !predicate(value));
 * }
 *
 * it('requires the predicate type to match the array type', () => {
 *   staticTest(() => {
 *     // @ts-expect-error -- mismatch of number array w/ string function
 *     reject([1, 2, 3], (value: string) => value === '2');
 *   });
 * }); * ```
 */
declare function staticTest(_test: VoidFunction): void;

export { AngularContext, AsyncMethodController, ComponentContext, ComponentHarnessSuperclass, IsPageVisibleHarness, SlTestRequest, TestCall, createSpyObject, expectCallsAndReset, expectRequest, expectSingleCallAndReset, logTimers, marbleTest, staticTest };
